"""Module to handle the history command."""

from argparse import Namespace
from pathlib import Path

from command_line_assistant.dbus.exceptions import (
    ChatNotFoundError,
    CorruptedHistoryError,
    HistoryNotAvailable,
    MissingHistoryFileError,
)
from command_line_assistant.integrations import BASH_INTERACTIVE
from command_line_assistant.rendering.renders.text import TextRenderer
from command_line_assistant.utils.cli import BaseCLICommand, SubParsersAction
from command_line_assistant.utils.environment import get_xdg_data_path
from command_line_assistant.utils.renderers import (
    create_error_renderer,
    create_text_renderer,
)


class ShellCommand(BaseCLICommand):
    """Class that represents the history command."""

    def __init__(self, args: Namespace) -> None:
        """Constructor of the class.

        Note:
            If none of the above is specified, the command will retrieve all
            user history.

        Arguments:
            args (Namespace): The args for that command.
        """
        self._args = args

        self._data_dir: Path = Path(get_xdg_data_path(), "command-line-assistant")

        self._text_renderer: TextRenderer = create_text_renderer()
        self._error_renderer: TextRenderer = create_error_renderer()

        super().__init__()

    def run(self) -> int:
        """Main entrypoint for the command to run.

        Returns:
            int: Status code of the execution.
        """
        try:
            if self._args.enable_integration:
                self._write_bash_functions()
            return 0
        except (
            MissingHistoryFileError,
            CorruptedHistoryError,
            ChatNotFoundError,
            HistoryNotAvailable,
        ) as e:
            self._error_renderer.render(str(e))
            return 1

    def _write_bash_functions(self) -> None:
        if not self._data_dir.exists():
            try:
                self._data_dir.mkdir()
            except FileExistsError:
                pass

        try:
            integration_file = Path(self._data_dir, "cla.bash")
            integration_file.write_text(BASH_INTERACTIVE)
        except FileExistsError:
            pass


def register_subcommand(parser: SubParsersAction):
    """
    Register this command to argparse so it's available for the root parser.

    Args:
        parser (SubParsersAction): Root parser to register command-specific arguments
    """
    shell_parser = parser.add_parser(
        "shell",
        help="Manage shell integrations",
    )
    shell_parser.add_argument(
        "--enable-integration",
        action="store_true",
        help="Enable the shell integrations on the system. Currently, only BASH is supported.",
    )

    shell_parser.set_defaults(func=_command_factory)


def _command_factory(args: Namespace) -> ShellCommand:
    """Internal command factory to create the command class

    Args:
        args (Namespace): The arguments processed with argparse.

    Returns:
        ShellCommand: Return an instance of class
    """
    return ShellCommand(args)
